# -*- coding: utf-8 -*-
""" =============================  Tsukuba Sawn oFF ============================
                        cut-down version to demonstrateminimum RT python-csound binding
For Thomas Hermann in memory of the train journey from Tokoyo to Tsukuba

csound/python process example for sound rendering control
Date: Created on Sonntag Oct  2015
Author: David Worrall david.worrall@anu.edu.au, david.worrall@fraunhofer.de, dworrall@olum.edu
Version: 1.0
2017091117:
    - alter final break from the while 1 loop
    - to execute using anaconda python with csound installed by the installer:
        import sys
        sys.path.append ('/LIbrary/python/2.7/site-packages') # where the csnd6.py module is held
20171009:
    sawn-off version to demonstrat eminimum RT python-csound binding for Data Sonification Class
20180605:
    ctcsound version: thanks François Pinot
"""


""" ======================= RENDER ORCHESTRA ==============================="""
# <begin copy from orchestra definitions file>
# NB use ''' not """ for these string variables
orchestraOptions='''
sr = 48000
kr = 4800
ksmps = 10
nchnls = 2
0dbfs = 1.0
'''
sineInstr_01='''
instr 1
      instrNr       = p1 ;;;;;;;;
      istartTime    = p2
	idur	= p3
	iamp	= p4
	ifreq	= p5
      iattk = p6
      isus  = p7
      idky = p8
      irel = p9
 	kloc=	 p10	; location

	kbeta	= 0		; for ambi out

; === AUDIO SYNTH ===
    k1 madsr iattk, isus,irel, idky
    asig oscili iamp, ifreq, 1
                                ;    k1 madsr 0.02, 0.1, iamp*0.5, 0.1
                                ;    asig oscils iamp, ifreq, 1
    asig = asig * k1


; === OUTPUT STAGE ===
    ; stereo out
    aleft, aright pan2 asig, (kloc % 180) * 180    ; stereo out with panning (0=hardL, 1 = hardR)
    outs aleft, aright
    endif
endin
'''
pluckInstr_02='''
instr 2
    instrNr       = p1
    istartTime    = p2
    kloc	= p9 ; location
    kbeta	= 0   ; spread? for ambi out

    ifn  = 0 ; random
	idur	= p3
    ifreq	= p4
    imeth	= p6
    imethP1	= p7
    imethP2	= p8

; === AUDIO SYNTH ===
  asig pluck ifreq, p5, p5, ifn, imeth, imethP1, imethP2
  kres line 1, idur, 0  ; mainly so tail does not leave DC offset hanging.
  asig = asig * kres

; === OUTPUT STAGE ===
    ; stereo out
    aL, aR pan2 asig, (kloc % 180) * 180    ; stereo out with panning (0=hardL, 1 = hardR)
    outs aL, aR
endin
'''
# <end copy from orchestra definitions file>

import ctcsound
cs=ctcsound.Csound()
cs.setOption('-odac')
cs.setOption('-b 4096')
csPerf = ctcsound.CsoundPerformanceThread(cs.csound()) # instantiate the RT perf thread
cs.readScore("f 0 z \n")                            # instantiate CS

cs.compileOrc(orchestraOptions)                     # SR, NrChans etc  # from sflowOrchestra.py

cs.start()                                          # start CS synth
csPerf.play()

cs.compileOrc(sineInstr_01)     # compile an instrument- defined in sflowOrchestra.py 
cs.compileOrc(pluckInstr_02)    # pluckInstr_02 is a variable name
# ....compile other instruments here....

#compile f statements
cs.readScore("f 1 0 16384 10 1")
cs.readScore("f 2 0 17 0 0 1 1")
cs.readScore("f31 0 8 2 1 0.6 10 100 0.001 ; 1 rattle")
cs.readScore("f32 0 8 2 1 0.7 50 500 1000  ; 1 rubber")

# to suppress ALL terminal print output from csound
global TRACE 
TRACE = False
""" #drop for the mo 20180606 
def passCallback():  pass
if not TRACE:
    cs.setMessageCallback(passCallback())
"""
# Generate code to send to RT csound, using a generic event generator
def makeCSevent (*args):
    """
    Makes a csound event and plays it in an already active Performance thread.
    Used to send strings of commands to csound. 
    """
    eventStr = ' '.join(map(str, args))
    print (eventStr)
    cs.readScore (eventStr) #    emit it!

global OP_EVENT_COUNT
OP_EVENT_COUNT = 0
import time, random

def playAlap():
    global OP_EVENT_COUNT
    OP_EVENT_COUNT += 1
    startTime  = 0
    amp = 1 
    azimuth = random.random()*180  # NB thus all notes in the nrNotes loop have same azimuth
    instrNr = 2 # pluck
    method = random.choice ([1, 3, ])   # 1 3 4 are possible
    mParam1 =  1                        # random.choice ([0.8, 0.9, 1])
    mParam2 =  random.choice([1, 10])
    print ("EVENT Nr: %i , Method: %i, P1: %2.1f, P2: %d"  % (OP_EVENT_COUNT, method, mParam1, mParam2))
    nrNotes = random.choice([3,5,8,13])
    fund = 55 * random.choice([7,8,9,10,11,12,13])
    startOffset = random.choice([x*0.167 for x in range(2)])
    for i in range(nrNotes):
        startOffset+=random.choice([0.167, 0.25, 0.25, 0.3])
        startOffset = startOffset +random.choice ([-0.01, 0, 0.01])
        startTime += startOffset *0.618
        freq = fund * random.choice([9/8.0, 8/9.0, 5/4.0, 4/5.0, 6/5.0, 5/6.0 ])
        durat = random.choice([1,2,3])
        makeCSevent ('i', instrNr, startTime, durat,  amp* 0.4, freq, method, mParam1,mParam2, azimuth)#, OP_EVENT_COUNT )
    time.sleep(startTime)

def playTsukubaExpress():
    global OP_EVENT_COUNT
    OP_EVENT_COUNT += 1

    startTime  = 0 
    amp = 1 
    azimuth = random.random()*180       # NB thus all notes in the nrNotes loop have same azimuth
    instrNr = 2 # pluck
    method = random.choice ([1, 3, ])   # 1 3 4 are possible
    mParam1 =  random.choice ([0.7, 0.8, 0.9, 1])
    mParam2 =  random.choice([1, 10])
    print ("EVENT Nr: %i , Method: %i, P1: %2.1f, P2: %d"  % (OP_EVENT_COUNT, method, mParam1, mParam2))
    nrNotes = random.choice([3,5,8,13])
    fund = 110 * random.choice([1,2,3,4,5,6,7])
    for i in range(nrNotes):
        startOffset = random.choice([0.167, 0.3])
        startOffset = startOffset +random.choice ([-0.01, 0, 0.01])
        startTime += startOffset *0.618
        freq = fund * random.choice([9/8.0, 8/9.0, 5/4.0, 4/5.0, 6/5.0, 5/6.0 ])
        durat = random.choice([1,2,3])
        makeCSevent ('i', instrNr, startTime, durat,  amp* 0.4, freq, method, mParam1,mParam2, azimuth) #, OP_EVENT_COUNT)
    time.sleep(startTime)

if not TRACE:
    print ("\n++++++++++++++++++++++++++++++++++++++++++++++++++++++++")
    print ("Ready to start the sonification from the user interface.")
    print ("++++++++++++++++++++++++++++++++++++++++++++++++++++++++")

while 1:
    if OP_EVENT_COUNT < 21:
        print ("playAlap" ,)
        playAlap()  # commenting out this line will cause no sound to be generated
    elif OP_EVENT_COUNT < 144:
        print ("playTsukubaExpress" ,)
        playTsukubaExpress()  # commenting out this line will cause no sound to be generated
    elif OP_EVENT_COUNT < 144+13:
        print ("playTsukubaExpress" ,)
        playAlap()  # commenting out this line will cause no sound to be generated
    elif OP_EVENT_COUNT > 144+13+1:
        print ("FIN")
 #       print ("FIN_01")
        break

while (c.performKsmps() == 0):
      pass

c.stop()
"""
cs.clean()
cs.stop()
if __name__ == "__main__":
#    import sys
#    sys.path
#    while 1:
    playTsukubaExpress() #doInner()
    cs.clean()
    cs.stop()
#      break
#    exit()
"""
